@@ -189,7 +189,13 @@ class SolrSearchBackend(BaseSearchBackend):
             kwargs['hl.fragsize'] = '200'
 
             if isinstance(highlight, dict):
-                kwargs.update(highlight)
+                # autoprefix highlighter options with 'hl.', all of them start with it anyway
+                # this makes option dicts shorter: {'maxAnalyzedChars': 42}
+                # and lets some of options be used as keyword arguments: `.highlight(preserveMulti=False)`
+                kwargs.update({
+                    key if key.startswith("hl.") else ('hl.' + key): highlight[key] 
+                    for key in highlight.keys()
+                })
 
         if self.include_spelling is True:
             kwargs['spellcheck'] = 'true'
@@ -565,7 +571,9 @@ class SolrSearchQuery(BaseSearchQuery):
             index_fieldname = u'%s:' % connections[self._using].get_unified_index().get_index_fieldname(field)
 
         filter_types = {
-            'contains': u'%s',
+            'content': u'%s',
+            'contains': u'*%s*',
+            'endswith': u'*%s',
             'startswith': u'%s*',
             'exact': u'%s',
             'gt': u'{%s TO *}',
@@ -578,7 +586,7 @@ class SolrSearchQuery(BaseSearchQuery):
         if value.post_process is False:
             query_frag = prepared_value
         else:
-            if filter_type in ['contains', 'startswith', 'fuzzy']:
+            if filter_type in ['content', 'contains', 'startswith', 'endswith', 'fuzzy']:
                 if value.input_type_name == 'exact':
                     query_frag = prepared_value
                 else:
