@@ -5,8 +5,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from django.conf import settings
 from django.core.paginator import InvalidPage, Paginator
 from django.http import Http404
-from django.shortcuts import render_to_response
-from django.template import RequestContext
+from django.shortcuts import render
 
 from haystack.forms import FacetedSearchForm, ModelSearchForm
 from haystack.query import EmptySearchQuerySet
@@ -23,10 +22,9 @@ class SearchView(object):
     form = None
     results_per_page = RESULTS_PER_PAGE
 
-    def __init__(self, template=None, load_all=True, form_class=None, searchqueryset=None, context_class=RequestContext, results_per_page=None):
+    def __init__(self, template=None, load_all=True, form_class=None, searchqueryset=None, results_per_page=None):
         self.load_all = load_all
         self.form_class = form_class
-        self.context_class = context_class
         self.searchqueryset = searchqueryset
 
         if form_class is None:
@@ -126,10 +124,7 @@ class SearchView(object):
         """
         return {}
 
-    def create_response(self):
-        """
-        Generates the actual HttpResponse to send back to the user.
-        """
+    def get_context(self):
         (paginator, page) = self.build_page()
 
         context = {
@@ -144,7 +139,17 @@ class SearchView(object):
             context['suggestion'] = self.form.get_suggestion()
 
         context.update(self.extra_context())
-        return render_to_response(self.template, context, context_instance=self.context_class(self.request))
+
+        return context
+
+    def create_response(self):
+        """
+        Generates the actual HttpResponse to send back to the user.
+        """
+
+        context = self.get_context()
+
+        return render(self.request, self.template, context)
 
 
 def search_view_factory(view_class=SearchView, *args, **kwargs):
@@ -178,7 +183,7 @@ class FacetedSearchView(SearchView):
         return extra
 
 
-def basic_search(request, template='search/search.html', load_all=True, form_class=ModelSearchForm, searchqueryset=None, context_class=RequestContext, extra_context=None, results_per_page=None):
+def basic_search(request, template='search/search.html', load_all=True, form_class=ModelSearchForm, searchqueryset=None, extra_context=None, results_per_page=None):
     """
     A more traditional view that also demonstrate an alternative
     way to use Haystack.
@@ -232,4 +237,4 @@ def basic_search(request, template='search/search.html', load_all=True, form_cla
     if extra_context:
         context.update(extra_context)
 
-    return render_to_response(template, context, context_instance=context_class(request))
+    return render(request, template, context)
