@@ -390,6 +390,22 @@ class SolrSearchBackendTestCase(TestCase):
         self.assertEqual(self.sb.search('Index', highlight=True)['hits'], 3)
         self.assertEqual([result.highlighted['text'][0] for result in self.sb.search('Index', highlight=True)['results']], ['<em>Indexed</em>!\n1', '<em>Indexed</em>!\n2', '<em>Indexed</em>!\n3'])
 
+        # shortened highlighting options
+        highlight_dict = {'simple.pre':'<i>', 'simple.post': '</i>'}
+        self.assertEqual(self.sb.search('', highlight=highlight_dict), {'hits': 0, 'results': []})
+        self.assertEqual(self.sb.search('Index', highlight=highlight_dict)['hits'], 3)
+        self.assertEqual([result.highlighted['text'][0] for result in self.sb.search('Index', highlight=highlight_dict)['results']],
+            ['<i>Indexed</i>!\n1', '<i>Indexed</i>!\n2', '<i>Indexed</i>!\n3'])
+
+        # full-form highlighting options
+        highlight_dict = {'hl.simple.pre':'<i>', 'hl.simple.post': '</i>'}
+        self.assertEqual([result.highlighted['text'][0] for result in self.sb.search('Index', highlight=highlight_dict)['results']],
+            ['<i>Indexed</i>!\n1', '<i>Indexed</i>!\n2', '<i>Indexed</i>!\n3'])
+
+        self.assertEqual(self.sb.search('Indx')['hits'], 0)
+        self.assertEqual(self.sb.search('indax')['spelling_suggestion'], 'index')
+        self.assertEqual(self.sb.search('Indx', spelling_query='indexy')['spelling_suggestion'], 'index')
+
         self.assertEqual(self.sb.search('', facets={'name': {}}), {'hits': 0, 'results': []})
         results = self.sb.search('Index', facets={'name': {}})
         self.assertEqual(results['hits'], 3)
@@ -957,7 +973,7 @@ class LiveSolrSearchQuerySetTestCase(TestCase):
         # This will break horrifically if escaping isn't working.
         sqs = self.sqs.auto_query('"pants:rule"')
         self.assertTrue(isinstance(sqs, SearchQuerySet))
-        self.assertEqual(repr(sqs.query.query_filter), '<SQ: AND content__contains="pants:rule">')
+        self.assertEqual(repr(sqs.query.query_filter), '<SQ: AND content__content="pants:rule">')
         self.assertEqual(sqs.query.build_query(), u'("pants\\:rule")')
         self.assertEqual(len(sqs), 0)
 
@@ -1217,12 +1233,10 @@ class LiveSolrMoreLikeThisTestCase(TestCase):
 
     def test_more_like_this_defer(self):
         mi = MockModel.objects.defer('foo').get(pk=1)
-        # FIXME: this currently is known to fail because haystack.utils.loading doesn't see the
-        #        MockModel_Deferred_foo class as registered:
         deferred = self.sqs.models(MockModel).more_like_this(mi)
-        self.assertEqual(deferred.count(), 0)
-        self.assertEqual([result.pk for result in deferred], [])
-        self.assertEqual(len([result.pk for result in deferred]), 0)
+        top_results = [int(result.pk) for result in deferred[:5]]
+        for i in (14, 6, 4, 22, 10):
+            self.assertIn(i, top_results)
 
     def test_more_like_this_custom_result_class(self):
         """Ensure that swapping the ``result_class`` works"""
